from __future__ import print_function

import os

import numpy as np
import skimage.io as io
import skimage.transform as trans
from keras.preprocessing.image import ImageDataGenerator


def adjustData_multi(img):
    img = img / 255
    return (img)


def trainGenerator_mlfdb(batch_size, train_path, aug_dict, image_color_mode="grayscale",
                         mask_color_mode="grayscale", image_save_prefix="image", mask_save_prefix="mask",
                         save_to_dir=None, input_size=(32, 32), output_size=(64, 64), seed=1, model_name="unet_1"):

    image_datagen = ImageDataGenerator(**aug_dict)

    image_generator_1 = image_datagen.flow_from_directory(
        train_path + "1",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_1",
        seed=seed)

    if not os.path.exists(model_name + '/train_1.txt'):
        with open(model_name + '/train_1.txt', 'w') as f:
            for item in image_generator_1.filenames:
                f.write("%s\n" % item)

    image_generator_2 = image_datagen.flow_from_directory(
        train_path + "2",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_2",
        seed=seed)

    if not os.path.exists(model_name + '/train_2.txt'):
        with open(model_name + '/train_2.txt', 'w') as f:
            for item in image_generator_2.filenames:
                f.write("%s\n" % item)

    image_generator_3 = image_datagen.flow_from_directory(
        train_path + "3",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_3",
        seed=seed)

    if not os.path.exists(model_name + '/train_3.txt'):
        with open(model_name + '/train_3.txt', 'w') as f:
            for item in image_generator_3.filenames:
                f.write("%s\n" % item)

    image_generator_4 = image_datagen.flow_from_directory(
        train_path + "4",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_4",
        seed=seed)

    if not os.path.exists(model_name + '/train_4.txt'):
        with open(model_name + '/train_4.txt', 'w') as f:
            for item in image_generator_4.filenames:
                f.write("%s\n" % item)

    image_generator_5 = image_datagen.flow_from_directory(
        train_path + "5",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_5",
        seed=seed)

    if not os.path.exists(model_name + '/train_5.txt'):
        with open(model_name + '/train_5.txt', 'w') as f:
            for item in image_generator_5.filenames:
                f.write("%s\n" % item)

    image_generator_6 = image_datagen.flow_from_directory(
        train_path + "6",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_6",
        seed=seed)

    if not os.path.exists(model_name + '/train_6.txt'):
        with open(model_name + '/train_6.txt', 'w') as f:
            for item in image_generator_6.filenames:
                f.write("%s\n" % item)

    image_generator_7 = image_datagen.flow_from_directory(
        train_path + "7",
        classes=['32x32'],
        class_mode=None,
        shuffle=False,
        color_mode=image_color_mode,
        target_size=input_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=image_save_prefix + "_7",
        seed=seed)

    if not os.path.exists(model_name + '/train_7.txt'):
        with open(model_name + '/train_7.txt', 'w') as f:
            for item in image_generator_7.filenames:
                f.write("%s\n" % item)

    mask_generator = image_datagen.flow_from_directory(
        train_path,
        classes=['output_64'],
        class_mode=None,
        shuffle=False,
        color_mode=mask_color_mode,
        target_size=output_size,
        batch_size=batch_size,
        save_to_dir=save_to_dir,
        save_prefix=mask_save_prefix,
        seed=seed)

    if not os.path.exists(model_name + '/train_mask.txt'):
        with open(model_name + '/train_mask.txt', 'w') as f:
            for item in image_generator_7.filenames:
                f.write("%s\n" % item)

    train_generator = zip(image_generator_1, image_generator_2, image_generator_3, image_generator_4, image_generator_5,
                          image_generator_6, image_generator_7, mask_generator)
    for (img_1, img_2, img_3, img_4, img_5, img_6, img_7, mask) in train_generator:
        img_1 = adjustData_multi(img_1)
        img_2 = adjustData_multi(img_2)
        img_3 = adjustData_multi(img_3)
        img_4 = adjustData_multi(img_4)
        img_5 = adjustData_multi(img_5)
        img_6 = adjustData_multi(img_6)
        img_7 = adjustData_multi(img_7)
        mask = adjustData_multi(mask)
        yield ([img_1, img_2, img_3, img_4, img_5, img_6, img_7], mask)


def get_images(test_path, file, input_size=(256, 256), as_gray=False):
    img_1 = io.imread(os.path.join(test_path + "1/32x32/" + file), as_gray=as_gray)
    img_1 = img_1 / 255
    img_1 = trans.resize(img_1, input_size)
    img_1 = np.reshape(img_1, (1,) + img_1.shape)

    img_2 = io.imread(os.path.join(test_path + "2/32x32/" + file), as_gray=as_gray)
    img_2 = img_2 / 255
    img_2 = trans.resize(img_2, input_size)
    img_2 = np.reshape(img_2, (1,) + img_2.shape)

    img_3 = io.imread(os.path.join(test_path + "3/32x32/" + file), as_gray=as_gray)
    img_3 = img_3 / 255
    img_3 = trans.resize(img_3, input_size)
    img_3 = np.reshape(img_3, (1,) + img_3.shape)

    img_4 = io.imread(os.path.join(test_path + "4/32x32/" + file), as_gray=as_gray)
    img_4 = img_4 / 255
    img_4 = trans.resize(img_4, input_size)
    img_4 = np.reshape(img_4, (1,) + img_4.shape)

    img_5 = io.imread(os.path.join(test_path + "5/32x32/" + file), as_gray=as_gray)
    img_5 = img_5 / 255
    img_5 = trans.resize(img_5, input_size)
    img_5 = np.reshape(img_5, (1,) + img_5.shape)

    img_6 = io.imread(os.path.join(test_path + "6/32x32/" + file), as_gray=as_gray)
    img_6 = img_6 / 255
    img_6 = trans.resize(img_6, input_size)
    img_6 = np.reshape(img_6, (1,) + img_6.shape)

    img_7 = io.imread(os.path.join(test_path + "7/32x32/" + file), as_gray=as_gray)
    img_7 = img_7 / 255
    img_7 = trans.resize(img_7, input_size)
    img_7 = np.reshape(img_7, (1,) + img_7.shape)
    return [img_1, img_2, img_3, img_4, img_5, img_6, img_7]


def saveResult_test(save_path, npyfile):
    for i, item in enumerate(npyfile):
        item[item > 1] = 1
        item[item <= 0] = 0
        rescaled = (item * 255).astype(np.uint8)
        io.imsave(os.path.join(save_path, "%d.JPG" % i), rescaled)
