from keras.callbacks import ModelCheckpoint

from models.data import *
from models.unet_geu_2.model import *  # Change the model

os.environ['KERAS_BACKEND'] = 'tensorflow'
os.environ['CUDA_DEVICE_ORDER'] = 'PCI_BUS_ID'
os.environ['CUDA_VISIBLE_DEVICES'] = '0'
seed_value = 0
# 1. Set the `PYTHONHASHSEED` environment variable at a fixed value
os.environ['PYTHONHASHSEED'] = str(seed_value)
# 2. Set the `python` built-in pseudo-random generator at a fixed value
import random

random.seed(seed_value)
# 3. Set the `numpy` pseudo-random generator at a fixed value
import numpy as np

np.random.seed(seed_value)
# 4. Set the `tensorflow` pseudo-random generator at a fixed value
import tensorflow as tf

tf.set_random_seed(seed_value)

### For UNET_1 input and output size are the same

weight_path = "unet_1/mlfdb_64.hdf5"
train_path = '../mlfdb_prepared/train/'
valid_path = '../mlfdb_prepared/test/'
in_size = 32
scale = 2
out_size = 64

data_gen_args_train = dict(rotation_range=0.1,
                           width_shift_range=0.1,
                           height_shift_range=0.1,
                           shear_range=0.1,
                           zoom_range=0.1,
                           horizontal_flip=False,
                           vertical_flip=False,
                           fill_mode='nearest',
                           )

data_gen_args_valid = dict(rotation_range=0.1,
                           width_shift_range=0.1,
                           height_shift_range=0.1,
                           shear_range=0.1,
                           zoom_range=0.1,
                           horizontal_flip=False,
                           vertical_flip=False,
                           fill_mode='nearest',
                           )

frames = trainGenerator_mlfdb(4, train_path, data_gen_args_train,
                              image_color_mode="rgb",
                              mask_color_mode="rgb",
                              input_size=(in_size, in_size),
                              model_name="unet_1/train_meta",
                              output_size=(out_size, out_size))

frames_valid = trainGenerator_mlfdb(4, valid_path, data_gen_args_valid,
                                    image_color_mode="rgb",
                                    mask_color_mode="rgb", input_size=(in_size, in_size),
                                    model_name="unet_1/valid_meta",
                                    output_size=(out_size, out_size))

model = network(scale=scale, input_size=(in_size, in_size, 3))
# model.load_weights(weight_path)
model_checkpoint = ModelCheckpoint(weight_path, verbose=1, monitor="val_acc", save_best_only=True)
his = model.fit_generator(frames,
                          steps_per_epoch=500,
                          epochs=500,
                          callbacks=[model_checkpoint],
                          validation_data=frames_valid,
                          validation_steps=4
                          )
