from models.help_functions import *
from models.loss_functions import *


def mergeNet(input, frame4):
    kernel = 3

    def resBlock(res_input):
        kernel = 3
        conv_1 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                        )(res_input)
        activation_1 = LeakyReLU(alpha=0.1)(conv_1)
        conv_2 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                        )(activation_1)
        add_3 = Add()([conv_2, res_input])

        return add_3

    conv0 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(input)
    resBlock_1 = resBlock(conv0)
    resBlock_2 = resBlock(resBlock_1)
    resBlock_3 = resBlock(resBlock_2)
    resBlock_4 = resBlock(resBlock_3)
    resBlock_5 = resBlock(resBlock_4)
    resBlock_6 = resBlock(resBlock_5)
    resBlock_7 = resBlock(resBlock_6)
    resBlock_8 = resBlock(resBlock_7)

    conv_fr4 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                      )(frame4)

    add_final = Add()([resBlock_8, conv_fr4])

    conv_final = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                        )(add_final)

    return conv_final


def single_unet(input):
    kernel = 3
    conv0 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(input)
    conv0 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv0)
    pool0 = MaxPooling2D(pool_size=(2, 2))(conv0)

    conv1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(pool0)
    conv1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv1)
    pool1 = MaxPooling2D(pool_size=(2, 2))(conv1)

    conv2 = Conv2D(128, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(pool1)
    conv2 = Conv2D(128, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv2)
    pool2 = MaxPooling2D(pool_size=(2, 2))(conv2)

    conv3 = Conv2D(256, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(pool2)
    conv3 = Conv2D(256, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv3)
    pool3 = MaxPooling2D(pool_size=(2, 2))(conv3)

    conv4 = Conv2D(512, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(pool3)
    conv4 = Conv2D(512, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv4)
    pool4 = MaxPooling2D(pool_size=(2, 2))(conv4)

    conv5 = Conv2D(1024, 3, activation='relu', padding='same', kernel_initializer='he_normal')(pool4)
    conv5 = Conv2D(1024, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv5)
    drop5 = Dropout(0.5)(conv5)

    up6 = upsample(drop5, 2, 512)
    merge6 = concatenate([conv4, up6], axis=3)
    conv6 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(merge6)
    conv6 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv6)

    up7 = upsample(conv6, 2, 256)
    merge7 = concatenate([conv3, up7], axis=3)
    conv7 = Conv2D(256, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(merge7)
    conv7 = Conv2D(256, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv7)

    up8 = upsample(conv7, 2, 128)
    merge8 = concatenate([conv2, up8], axis=3)
    conv8 = Conv2D(128, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(merge8)
    conv8 = Conv2D(128, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv8)

    up9 = upsample(conv8, 2, 64)
    merge9 = concatenate([conv1, up9], axis=3)
    conv9 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(merge9)
    conv9 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                   )(conv9)

    up10 = upsample(conv9, 2, 32)
    merge10 = concatenate([conv0, up10], axis=3)
    conv10 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                    )(merge10)
    conv10 = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                    )(conv10)
    conv11 = Conv2D(3, 1)(conv10)

    return conv11


def network(input_size=(64, 64, 3), scale=2):
    kernel = 3
    input_1 = Input(input_size)
    conv0_1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_1)
    activation1 = LeakyReLU(alpha=0.2)(conv0_1)
    conv0_1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation1)
    activation1 = LeakyReLU(alpha=0.2)(conv0_1)
    conv0_1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation1)
    activation1 = LeakyReLU(alpha=0.2)(conv0_1)

    input_2 = Input(input_size)
    conv0_2 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_2)
    activation2 = LeakyReLU(alpha=0.2)(conv0_2)
    conv0_2 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation2)
    activation2 = LeakyReLU(alpha=0.2)(conv0_2)
    conv0_2 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation2)
    activation2 = LeakyReLU(alpha=0.2)(conv0_2)

    input_3 = Input(input_size)
    conv0_3 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_3)
    activation3 = LeakyReLU(alpha=0.2)(conv0_3)
    conv0_3 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation3)
    activation3 = LeakyReLU(alpha=0.2)(conv0_3)
    conv0_3 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation3)
    activation3 = LeakyReLU(alpha=0.2)(conv0_3)

    input_4 = Input(input_size)
    conv0_4 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_4)
    activation4 = LeakyReLU(alpha=0.2)(conv0_4)
    conv0_4 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation4)
    activation4 = LeakyReLU(alpha=0.2)(conv0_4)
    conv0_4 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation4)
    activation4 = LeakyReLU(alpha=0.2)(conv0_4)

    input_5 = Input(input_size)
    conv0_5 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_5)
    activation5 = LeakyReLU(alpha=0.1)(conv0_5)
    conv0_5 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation5)
    activation5 = LeakyReLU(alpha=0.1)(conv0_5)
    conv0_5 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation5)
    activation5 = LeakyReLU(alpha=0.2)(conv0_5)

    input_6 = Input(input_size)
    conv0_6 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_6)
    activation6 = LeakyReLU(alpha=0.2)(conv0_6)
    conv0_6 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation6)
    activation6 = LeakyReLU(alpha=0.2)(conv0_6)
    conv0_6 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation6)
    activation6 = LeakyReLU(alpha=0.2)(conv0_6)

    input_7 = Input(input_size)
    conv0_7 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(input_7)
    activation7 = LeakyReLU(alpha=0.2)(conv0_7)
    conv0_7 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation7)
    activation7 = LeakyReLU(alpha=0.2)(conv0_7)
    conv0_7 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal',
                     )(activation7)
    activation7 = LeakyReLU(alpha=0.2)(conv0_7)

    combined = concatenate([activation1, activation2, activation3, activation4, activation5, activation6, activation7])

    mergenet = mergeNet(combined, input_4)

    nn = single_unet(mergenet)

    model = Model(inputs=[input_1, input_2, input_3, input_4, input_5, input_6, input_7], outputs=nn)
    model.compile(optimizer=Adam(lr=1e-4), loss="mse", metrics=['mse', 'acc'])
    model.summary()

    return model
