from models.loss_functions import *
from models.help_functions import *


def pixel_shuffle(scale):
    return lambda x: tf.nn.depth_to_space(x, scale)


def PSNR(y_true, y_pred):
    # Compute PSNR over tf.uint8 Tensors.
    return tf.image.psnr(y_true, y_pred, max_val=1.0)


def geu_block(input):
    kernel = 3
    nn = Dropout(0.5)(input)
    nn = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(nn)
    nn = Activation('relu')(nn)

    nn = Conv2D(32, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(nn)
    nn = Activation('relu')(nn)

    conv_3 = Conv2D(3, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(nn)

    nn = concatenate([conv_3, input])

    nn = Conv2D(3, 1, activation='relu', padding='same', kernel_initializer='he_normal')(nn)
    nn = Activation("relu")(nn)

    nn = Lambda(lambda x: x[1] * x[2] + (1 - x[1]) * x[0])([input, nn, conv_3])

    return nn


def single_unet(inputs):
    kernel = 3
    conv1 = Conv2D(64, 3, activation='relu', padding='same', kernel_initializer='he_normal')(inputs)
    conv1 = Conv2D(64, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv1)
    pool1 = MaxPooling2D(pool_size=(2, 2))(conv1)
    conv2 = Conv2D(128, 3, activation='relu', padding='same', kernel_initializer='he_normal')(pool1)
    conv2 = Conv2D(128, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv2)
    pool2 = MaxPooling2D(pool_size=(2, 2))(conv2)
    conv3 = Conv2D(256, 3, activation='relu', padding='same', kernel_initializer='he_normal')(pool2)
    conv3 = Conv2D(256, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv3)
    pool3 = MaxPooling2D(pool_size=(2, 2))(conv3)
    conv4 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(pool3)
    conv4 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv4)
    drop4 = Dropout(0.5)(conv4)
    pool4 = MaxPooling2D(pool_size=(2, 2))(drop4)

    conv5 = Conv2D(1024, 3, activation='relu', padding='same', kernel_initializer='he_normal')(pool4)
    conv5 = Conv2D(1024, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv5)
    drop5 = Dropout(0.5)(conv5)

    up6 = Conv2D(512, 2, activation='relu', padding='same', kernel_initializer='he_normal')(
        UpSampling2D(size=(2, 2))(drop5))
    # up6 = upsample(drop5, 2, 512)
    merge6 = concatenate([drop4, up6], axis=3)
    conv6 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(merge6)
    conv6 = Conv2D(512, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv6)

    up7 = Conv2D(256, 2, activation='relu', padding='same', kernel_initializer='he_normal')(
        UpSampling2D(size=(2, 2))(conv6))
    # up7 = upsample(conv6, 2, 256)
    merge7 = concatenate([conv3, up7], axis=3)
    conv7 = Conv2D(256, 3, activation='relu', padding='same', kernel_initializer='he_normal')(merge7)
    conv7 = Conv2D(256, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv7)

    up8 = Conv2D(128, 2, activation='relu', padding='same', kernel_initializer='he_normal')(
        UpSampling2D(size=(2, 2))(conv7))
    # up8 = upsample(conv7, 2, 128)
    merge8 = concatenate([conv2, up8], axis=3)
    conv8 = Conv2D(128, 3, activation='relu', padding='same', kernel_initializer='he_normal')(merge8)
    conv8 = Conv2D(128, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv8)

    up9 = Conv2D(64, 2, activation='relu', padding='same', kernel_initializer='he_normal')(
        UpSampling2D(size=(2, 2))(conv8))
    # up9 = upsample(conv8, 2, 64)
    merge9 = concatenate([conv1, up9], axis=3)
    conv9 = Conv2D(64, 3, activation='relu', padding='same', kernel_initializer='he_normal')(merge9)
    conv9 = Conv2D(64, 3, activation='relu', padding='same', kernel_initializer='he_normal')(conv9)
    conv11 = Conv2D(3, 1)(conv9)

    return conv11


def network(input_size=(32, 32, 3), scale=2):
    kernel = 7

    input_1 = Input(input_size)
    conv0_1 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_1)
    activation1 = Activation('relu')(conv0_1)

    input_2 = Input(input_size)
    conv0_2 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_2)
    activation2 = Activation('relu')(conv0_2)

    input_3 = Input(input_size)
    conv0_3 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_3)
    activation3 = Activation('relu')(conv0_3)

    input_4 = Input(input_size)
    conv0_4 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_4)
    activation4 = Activation('relu')(conv0_4)

    input_5 = Input(input_size)
    conv0_5 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_5)
    activation5 = Activation('relu')(conv0_5)

    input_6 = Input(input_size)
    conv0_6 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_6)
    activation6 = Activation('relu')(conv0_6)

    input_7 = Input(input_size)
    conv0_7 = Conv2D(64, kernel, activation='relu', padding='same', kernel_initializer='he_normal')(input_7)
    activation7 = Activation('relu')(conv0_7)

    concat = concatenate([activation1, activation2, activation3, activation4, activation5, activation6, activation7],
                         axis=3)

    nn = Conv2D(3, 1, activation='relu', padding='same', kernel_initializer='he_normal')(concat)

    if scale == 2:
        nn = upsample(nn, 2, 64)
    elif scale == 4:
        nn = upsample(nn, 2, 64)
        nn = upsample(nn, 2, 64)
    elif scale == 8:
        nn = upsample(nn, 2, 64)
        nn = upsample(nn, 2, 64)
        nn = upsample(nn, 2, 64)

    nn = Conv2D(3, 1, strides=1, padding='same')(nn)

    nn = geu_block(nn)
    nn = geu_block(nn)
    nn = geu_block(nn)
    nn = geu_block(nn)
    nn = geu_block(nn)

    nn = single_unet(nn)

    model = Model(inputs=[input_1, input_2, input_3, input_4, input_5, input_6, input_7], outputs=nn)
    model.compile(optimizer=Adam(lr=1e-4), loss=vgg_19,
                  metrics=['mse', 'acc', PSNR])
    model.summary()

    return model
